/*
  otak project.
  copyright (C) 2002, 2003, 2004 
  grzegorz moskal,  <eldevarth@nemerle.org>
  license : gnu gpl version 2 			*/

#define EMAILOPT_C_
#include "emailopt.h"

typedef enum WORD_TYPE { 
	NORMAL = 1,
	EMAIL, 
	COMMENT 
} word_type;

struct word;
struct word {
	word_type type;
	char *caption;
	struct word *next;
};
static struct word *first_word, *last_word;

static int quota(int c)
{
	static int q = 0, bn = 0, bs = 0;
	
	switch (c) {
	case '\"':
		q = q ? 0 : 1;
		break;
	case '(':
		bn++;
		break;
	case ')':
		bn--;
		break;
	case 0:
		q = bs = bn = 0;
		break;
	}
	return q + bs + bn;
}

static int strchrs(char *s, char *chars)
{
	int i = strlen(chars)-1;
	for (; i+1; i--)
		if (strchr(s, chars[i]))
			return 1;
	return 0;
}

static word_type return_type(char *s)
{
	if (strchrs(s, "<>@"))
		return EMAIL;
		
	else if (strchrs(s,"()\""))
		return COMMENT;
		
	return NORMAL;
}

static char *parse_word(char *src)
{
	char *forb = "<>\"()", *src2, *j1, *j2;
	
	if (!src)
		return NULL;

	for (j1 = src, j2 = src2 = strdup(src); *j1;) {
		if (strchr(forb, *j1)) {
			j1++;
			continue;
		}
		*j2++ = *j1++;
	}

	for (xfree(src), *j2 = '\0'; *src2 && isspace(*src2); src2++); 
	for (j2 = src2+strlen(src2)-1; isspace(*j2) && j2 != src2;)
		*j2-- = '\0';
		
	return src2; 
}

static void add_word(char *s)
{
	struct word *w = xmalloc(sizeof(struct word));
	w->caption = s;
	w->type = return_type(s);
	w->next = 0;
	
	if (first_word) {
		last_word->next = w;
		last_word = w;
	} else
		last_word = first_word = w;
}

static char *get_word(char *str)
{
	char *w;
	int len = 0;
	
	for (w = str, quota(0); *w; w++) 
		if (!quota(*w))
			if (isspace(*w))
				break;
	
	len = (int)(w-str);
	w = xmalloc(sizeof(char) * (len +1));
	w[len] = '\0';
	
	return strncpy(w, str, len);
}

static void init_words(char *line)
{
	char *tmp;
	for (line = strchr(line, ':') + 1; isspace(*line); line++);
	do {
		tmp = get_word(line);
		add_word(tmp);
		for (line += strlen(tmp); *line && isspace(*line); line++);
	} while (line && *line);
}

static void free_words()
{
	struct word *j;
	for (j = first_word; first_word; ) {
		j = first_word;
		first_word = j->next;
		xfree(j->caption);
		xfree(j);
	}
}

static struct word *prev_word(struct word *w)
{
	struct word *j = first_word;
	for (; j->next != w; j = j->next);
	return j;
}

static struct word *return_word(struct word *begin, word_type type)
{
	for (; begin; begin = begin->next)
		if (begin->type == type)
			return begin;
	return 0;
}

static char *return_string(struct word *begin, struct word *end)
{
	char *ret = NULL, *jmp;
	struct word *j = begin;
	int len = 1;
	
	if (!begin || !end)
		return ret;

	for (; end->next != j; j = j->next) 
		len += strlen(j->caption) +1;
	
	ret = xmalloc(sizeof(char) * len);
	
	for (jmp = ret, j = begin; end->next != j; j = j->next) {
		sprintf(jmp, "%s ", j->caption);
		jmp += strlen(j->caption) + 1;
	}
	ret[len-1] = '\0';
	
	return ret;
}

static void set_name(char **name, struct word *wj)
{
	if (wj == first_word) {
		if ((wj = return_word(wj, COMMENT))) {
			if (first_word->next != wj)
				*name = return_string(first_word->next, wj);
			else
				*name = strdup(wj->caption);
		} else if ((wj = return_word(first_word, NORMAL))) {
			*name = strdup(wj->caption);	
			// *name = return_string(wj, last_word); // as u wish;)
		}
	} else  
		*name = return_string(first_word, prev_word(wj));
}

static void get_name(char *line, char **email, char **name)
{
	struct word *wj;

	*name =  NULL;
	*email = NULL;
	
	init_words(line);
	
	if ((wj = return_word(first_word, EMAIL))) {
		*email = strdup(wj->caption);
		set_name(name, wj);	
		
	} else if (first_word) {
		if ((wj = return_word(first_word, COMMENT)))
			*name = strdup(wj->caption);
		else
			*name = strdup(first_word->caption);
	}
	
	*name = parse_word(*name);
	*email = parse_word(*email);

	free_words();
}

static struct stanza *stanza_exist(char *name)
{
	struct stanza *s = find_stanza_by_name(name);
	if (!s)
		msgi("baseconfig doesn`t contains stanza (field) %s", name);
	return s;
}
static int confirm_adding_email()
{
	FILE *in;
	char *file;
	char *yes = translate("yes"), *no = translate("no");
	
	if (confirm_adding == 0)
		return 1;
		
	if ((in = fopen("/dev/tty", "r")) == 0) {
		errmsgf("Cannot open /dev/tty.", 0);
		errmsgf("You may want to use --add-email-quiet", 0);
		return 0;
	}
	
	file = (base_name) ? base_name : current_base->name;
			
	if (confirm(strglue(":: add to", " ", file), *yes, *no, in) == 0)
		return 0;
		
	return 1;
}
int try_add_email()
{
	char *name, *email;
	struct stanza *sname = stanza_exist("name");
	struct stanza *semail = stanza_exist("email");
	
	if (!new_email_arg) 
		return 0;
	
	get_name(new_email_arg, &email, &name);	

	if (!email || !sname)
		return 2;

	msgi(":: name  %s", name);
	msgi(":: email %s", email);
	
	if (confirm_adding_email() == 0) {
		msgi(":: %s", "aborted");
		return 3;
	}

	add_new_man();
	add_drawer(last_man, semail, email);
	add_drawer(last_man, stanza_exist("name"), name);
	msgi(":: %s", "done");

	return 1;
}

static void query_database(struct man *mj, int all)
{
	struct drawer *dmail, *dname;
	
	for (; mj; mj = mj->next) {
		if (!(dmail = find_drawer_by_name(mj, query_show)))
			continue;
		if (query_verify(query_arg, mj->first) == 0 && all == 0)
			continue;
		dname = find_drawer_by_name(mj, query_arg);
		if (dname == 0)
			dname = mj->first;
		query(dname->first->caption, dmail->first);
	}
}

int try_show_query()
{
	int all = (query_arg && !strcmp(query_arg, "a") ? 1 : 0);
	char **jmp;
	
	if (!(query_arg = strtolower(query_arg)))
		return 0;
		
	if (!stanza_exist(query_show)) 
		return 1;
		
	if (query_each_database) {
		struct base *m_base = current_base;
		struct man *m_man = current_man;
		
		for (current_base = first_base; current_base; SWITCH(current_base)) {
			current_man = current_base->first;
			query_database(current_man, all);
		}

		current_base = m_base;
		current_man = m_man;
	} else
		query_database(first_man, all);
	
	if (all)
		printf(translate(":: all items\n"));
		
	else if (query_counter)
		printf(translate(":: found %d item%s\n"), 
			query_counter, (query_counter-1) ? "s" : "" );
	else
		printf(translate(":: no items found\n")); 

	for (jmp = query_list; jmp && *jmp; jmp++)
		printf("%s\n", *jmp);

	query_free();	
	xfree(query_arg);
	return 2;
}
