/*
  otak project.
  copyright (C) 2002, 2003, 2004 
  grzegorz moskal,  <eldevarth@nemerle.org>
  license : gnu gpl version 2 			*/

#define MISCFNC_C_
#include "miscfnc.h"

void call_cmd(char *bin)
{	
	int pid, len, p[2], status;
	char buf[900], *j;

	pipe(p);
	pid = fork();

	if (pid == 0) {
		close(p[0]);
		
		dup2(p[1], 2);
		execl("/bin/sh", "/bin/sh", "-x", "-c", bin, NULL);
		errexit("Exec of /bin/sh failed ! Get conforming system\n");
		
	} else {
		long width;
		winfo_fill(wide_text("working ... "));
		stream(tsys_process(NULL));
		close(p[1]);
		
		strcpy(buf, "read failed");
		j = buf;
		winfo_get(TOBJ_WIDTH, &width);
		do {
			len = read(p[0], j, width-6);
			j += len+1;
			sprintf(j-1, "\n");
			if (j > buf + sizeof(buf) - 2*width)
				j = buf + sizeof(buf) - 2*width;
		} while (len > 0);
		
		wait(&status);
		tsys_domethod(TSYS_FORCE_REFRESH, 0);
		winfo_fill(strdup(buf));
	}
}

char *rline(FILE *fd)
{
	int i = 0, w = 6;
	char *tab = xmalloc(sizeof(char) * w);
	do {
		if (i >= w-2)
			tab = xrealloc(tab,(w*=2)*(sizeof(char)));
		if ((tab[i] = fgetc(fd)) == EOF) 
			return 0;
	} while (tab[i] != '\n' && ++i);
	tab[i] = 0;
	return i ? tab : rline(fd);
}

int confirm(char *question, char yes, char no, FILE *in)
{
	int c;
	char alt[9];

	sprintf(alt, " [%c/%c] ?", yes, no);
	msgi(strglue(question, " ", alt), 0);
	printf("\t :: ");
	fflush(stdout);
	do {
		c = fgetc(in);
		if (c == no) {
			fclose(in);
			return 0;
		}
	} while (c != yes);
	
	fclose(in);
	
	return 1;
		
}

char *return_otakvmsg(char *msg)
{
	if (msg == NULL)
		msg = "";
	return strglue(msg, program_name, "");
}

char *wide_text(char *s)
{
	char *wt = xmalloc(sizeof(char)*(strlen(s)*2+1));
	char *j = wt;
	while (s && *s) {
		*j++ = *s++;
		*j++ = ' ';
	}
	*j = '\000';
	return wt;
}

#define MSG_COLOR  36
void msgi(char *s, char *arg)
{
	if (!s) 
		return;
	printf("\t[%dm ", MSG_COLOR);
	printf(translate(s), translate(arg));
	printf("[0m\n");
}

static void msg(char *s, char *arg)
{
	if (!s)
		return;
	if (*s != '\n' && *s != '\0')
		printf("  [%dm-[1m-> [0m[%dm", MSG_COLOR, MSG_COLOR);
	printf(translate(s), translate(arg));
	printf("[0m\n");
}

// if silent msgf do nothing
void msgf(char *s, char *arg)
{
	if (silent)
		return;
	msg(s, arg);
}

void errmsgf(char *s, char *arg)
{
	if (silent == MESSAGES_AND_ERRORS)
		return;
	msg(s, arg);
}

void errexit(char *s)
{
	errmsgf(translate("error :"), s);
	exit(127);
}
/*
 *   Like cputs in conio.h, but with feature to dynamicly change
 *   colors of printed text. Using %f.ba in string, where f and b
 *   are numbers of colors listed below and:
 *   0  black   *   1  red 	 *   2  green  *   3  brown 
 *   4  blue    *   5  magenta   *   6  cyan   *   7  white 			
 *   ex. %4.0a - blue on black		
 *   use 'l' instead of 'a' to gain ligthcolor
 *   ex  %2.0l -  lightgreen on black					
 */

static char *cmsg_attrs(char *string)
{
	char t[10], *tmp;
	signed int fg = -1, bg = -1;
	tmp = t;

	while (*string <= '9' && *string >= '0')
		*(tmp++) = *(string++);

	if (tmp - t > 0)
		fg = atoi(t);

	if (*string == '.') {
		string++;
		tmp = t;
		while (*string <= '9' && *string >= '0')
			*(tmp++) = *(string++);

		if (tmp - t > 0)
			bg = atoi(t);
	}

	if ((*string == 'a' || *string == 'l') && isatty(1)) {
		printf("[0m");
		if (*string == 'l')
			printf("[1m");
		if (fg != -1) 
			printf("[%dm", fg+30);
		if (bg != -1)
			printf("[%dm", bg+40);
	}

	return string+1;
}

void cmsg(char *string)
{
	string = translate(string);
	if (isatty(1))
		printf("[0m");
	while (*string) {
		if ((*string) == '%') {
			string++;
			if ((*string) != '%')
				string = cmsg_attrs(string);
			else
				putchar('%');
		} else
			putchar(*(string++));
	}
	if (isatty(1))
		printf("[0m");
	printf("\n");
}

char *strglue(char *p1, char *space, char *p2)
{
	char *r = malloc(sizeof(char) * 
			(strlen(p1) + strlen(p2) + strlen(space) +1));
	sprintf(r,"%s%s%s", p1, space, p2);
	return r;
}

char *strdglue(char *p1, char *p2)
{
	return strglue(p1, "/", p2);
}

char *strtolower(char *str)
{
	char *ret;
	if (!str)
		return 0;

	ret = strdup(str);
	for (str = ret; *ret; ret++)
		*ret = tolower(*ret);
	return str;
}


char *xgetcwd()
{
	char *dir = NULL;
	int size = 100;

	dir = malloc(size);
	
	while (getcwd(dir, size) == NULL && errno == ERANGE)
		dir = xrealloc(dir, size *=2);
	return dir;
}

void *xfree(void *ptr)
{
	if (ptr)
		free(ptr);
	return 0;
}

void *xmalloc(size_t size)
{
	void *ptr;
	
	if (!size)
		return NULL;

	if ((ptr = calloc(1, size)) == NULL) {
		perror("calloc() failed");
		exit(1);
	}
	return ptr;
}

void *xrealloc(void *ptr, size_t size)
{

	if (!size)
		return NULL;
		
	ptr = realloc(ptr, size);
	
	if (ptr == NULL) {
		perror("realloc() failed");
		exit(1);
	}
	return ptr;
}

int d(int hm)
{
	if (hm)
		return (rand() % hm) + 1;

	abort();
}

void init_rand()
{
	srand(time(0) ^ getpid());
}

static void copy_file(char *sn, char *dn)
{
	FILE *src = fopen(sn, "r");
	FILE *dest = fopen(dn, "w+");
	if (src) {
		char *buf = xmalloc(sizeof(char)*200);
		*buf = 0;
		while ((buf = fgets(buf, 200, src)))
			fputs(buf, dest);
		xfree(buf);
		fclose(src);
	}
	fclose(dest);
}

void init_database()
{
	FILE *fd;
	baseconfig_name = strdglue(getenv("HOME"),".otak/baseconfig");
	basedir_name = strdglue(getenv("HOME"),".otak/base-dir");
	size_name = strdglue(getenv("HOME"),".otak/wins");

	
	if (!mkdir(strdglue(getenv("HOME"),".otak"), 0777)) {
		first_time = 1;
//		copy_file("/etc/otak/baseconfig", baseconfig_name);	// [that was stuped]
		copy_file(0, size_name);
		show_intro = 1;
	} else if (!(fd = fopen(size_name, "r")))
		copy_file(0, size_name);
		

	if (mkdir(strdglue(getenv("HOME"),".otak/base-dir"), 0777) == 0) 
		copy_file(strdglue(getenv("HOME"), ".otak/base"),
			  strdglue(getenv("HOME"), ".otak/base-dir/main"));
}

