#include "tsys.h"
#include "tobj.h"
#include "twin.h"
/* defines of events and methods */
#include "tinput.h"
#include <string.h>

#include "prvclass/tinput.h"

#define INSERT 0
#define REPLACE 1


/* moving cursor */
static void tinput_cursmove(struct tobj *obj)
{
	struct tinput *input;
	input = (struct tinput *) obj->data;

	if (tobj_isactive(obj)) {
		if (input->inputmode == INSERT)
			t_cursortype(TSCR_LINECURSOR);
		else
			t_cursortype(TSCR_SOLIDCURSOR);

		t_cursormove(tobj_getx(obj) +
			     MIN(input->curspos, obj->w - 1), tobj_gety(obj));
	}
}
void tinput_dorefresh(struct tobj *obj)
{
	struct tinput *input;
	unsigned int i, len;
	char c;

	input = (struct tinput *) obj->data;

	if (tobj_isactive(obj))
		t_textcolor(obj->fgcolor);
	else
		t_textcolor(obj->fgcolor2);
	t_textbackground(obj->bgcolor);


	/* setting proper start char
	   (so correct part of string will be showed) */
	if (input->curspos >= obj->w - 1)
		i = input->curspos - (obj->w - 1);
	else
		i = 0;

	len = strlen(input->showed);
	while (i <= MAX(input->curspos, obj->w - 1)) {
		if (i < len) {
			c = input->showed[i];
			if (c != '\n' && c != '\0')
				t_putchar(c);
			else
				t_putchar(' ');
		} else
			t_putchar(' ');

		i++;
	}
	tinput_cursmove(obj);

}

/* inserts given char to buffer according insertmode etc. */
static void tinput_edittext(struct tobj *obj, int keycode)
{
	struct tinput *input;
	unsigned int len;
	input = (struct tinput *) obj->data;

	if (input->acceptchars && keycode != K_DEL && keycode != K_BS) {
		char *s;
		s = input->acceptchars;
		while (*s) {
			if ((*s) == (char) keycode)
				break;
			s++;
		}
		/* char wasn't found in accept char list */
		if (*s == '\0')
			return;
	} else if (input->rejectchars && keycode != K_DEL && keycode != K_BS) {
		char *s;
		s = input->rejectchars;
		while (*s) {
			if ((*s) == (char) keycode)
				return;
			s++;
		}
	}

	/* if user was looking into history and started to
	   edit some entry, we change edit-buffer
	   to that entry  */
	if (input->edited != input->showed) {
		strcpy(input->edited, input->showed);

		input->buf_edit = input->buf_show;
		input->showed = input->edited;
	}


	len = strlen(input->edited);


	if (keycode == K_DEL) {
		if (len > input->curspos) {
			memmove(&(input->edited[input->curspos]),
				&(input->edited[input->curspos + 1]),
				len - input->curspos);
		}
	} else if (keycode == K_BS) {
		if (input->curspos < len && input->curspos > 0) {
			memmove(&(input->edited[input->curspos - 1]),
				&(input->edited[input->curspos]),
				len - input->curspos);
			input->edited[len - 1] = '\0';
			input->curspos -= 1;
		} else if (input->curspos == len && input->curspos > 0) {
			input->edited[input->curspos - 1] = '\0';
			input->curspos -= 1;
		}
	} else if (input->inputmode == REPLACE) {
		if (input->curspos < len) {
			input->edited[input->curspos] = keycode;
			input->curspos += 1;
		} else if (len < input->maxlen) {
			input->edited[input->curspos] = keycode;
			input->edited[input->curspos + 1] = '\0';
			input->curspos += 1;
		}
	} else if (input->inputmode == INSERT) {
		if (input->curspos < len && len < input->maxlen) {
			memmove(&(input->edited[input->curspos + 1]),
				&(input->edited[input->curspos]),
				len + 1 - input->curspos);
			input->edited[input->curspos] = keycode;
			input->curspos += 1;
		} else if (input->curspos < len && len >= input->maxlen) {
			input->edited[input->curspos] = keycode;
			input->curspos += 1;
		} else if (len < input->maxlen) {
			input->edited[input->curspos] = keycode;
			input->edited[input->curspos + 1] = '\0';
			input->curspos += 1;
		}
	}
	tevent_send(obj, EVENT_TYPE_USER, EVENT_TINPUT_CHANGED);
	tobj_dorefresh(obj);

}

/* Reallocing history when progammer wants to change history size */
static int tinput_reallochistory(struct tobj *obj, unsigned int newsize)
{
	struct tinput *input;
	input = obj->data;

	/* if new size is bigger then old one */
	if (input->maxhistory > newsize) {
		unsigned int i;
		i = newsize + 1;
		while (i <= input->maxhistory) {
			if (input->text[i]) {
				free(input->text[i]);
				input->text[i] = NULL;
			}
			i++;
		}

		/* correcting some vars */
		if (input->numhist > newsize)
			input->numhist = newsize;
		if (input->buf_show > newsize) {
			input->buf_show = newsize;
			/* if inputline is showing something from history */
			if (input->showed != input->edited) {
				input->showed = input->text[input->buf_show];
				tobj_dorefresh(obj);
			}
		}
		if (input->buf_edit > newsize)
			input->buf_edit = newsize;

	}

	/* if sizes don't match realloc input->text */
	if (input->maxhistory != newsize) {
		char **tmp;
		if ((tmp =
		     realloc(input->text, (newsize + 1) * sizeof(char *)))) {
			input->text = tmp;
			input->maxhistory = newsize;
		} else
			return 0;

	}
	return 1;

}
static int tinput_set(struct tobj *obj, struct tobj_set *tos)
{
	struct tinput *input;
	input = obj->data;

	switch (tos->attr) {
	case TINPUT_BUFFER_CURRENT:
		{
			unsigned int len;
			strcpy(input->edited, (char *) tos->data);
			len = strlen(input->edited);

			input->showed = input->edited;
			if (input->curspos > len)
				input->curspos = len;

			tobj_dorefresh(obj);
			break;
		}
	case TINPUT_CURSOR_POS:{
			int len;
			len = strlen(input->showed);
			input->curspos = MAX(0, MIN(len, tos->data));
			tobj_dorefresh(obj);
			break;
		}
	case TINPUT_INPUTMODE:
		input->inputmode =
		    (tos->data == TINPUT_INPUTMODE_INSERT) ? INSERT : REPLACE;
		tinput_cursmove(obj);
		break;
	case TINPUT_HISTORY_MAXLEN:
		if (!(tinput_reallochistory(obj, tos->data)))
			tsys_error_exit
			    ("ERROR: While reallocating history buffers\n");
		break;
	case TINPUT_ACCEPT_CHARS:
		if (input->acceptchars)
			free(input->acceptchars);

		if (tos->data) {
			if (!(input->acceptchars = strdup((char *) tos->data)))
				tsys_error_exit
				    ("ERROR: While changing accept chars list in tinput.\n");
		} else
			input->acceptchars = NULL;
		break;
	case TINPUT_REJECT_CHARS:
		if (input->rejectchars)
			free(input->rejectchars);

		if (tos->data) {
			if (!(input->rejectchars = strdup((char *) tos->data)))
				tsys_error_exit
				    ("ERROR: While changing reject chars list in tinput.\n");
		} else
			input->rejectchars = NULL;
		break;
	default:
		return 0;
	}
	return 1;
}

static int tinput_get(struct tobj *obj, struct tobj_get *tog)
{
	struct tinput *input;
	input = obj->data;

	switch (tog->attr) {
	case TINPUT_BUFFER_CURRENT:
		*(tog->data) = TPDATA(input->showed);
		break;
	case TINPUT_BUFFER_MAXLEN:
		*(tog->data) = input->maxlen;
		break;
	case TINPUT_BUFFER_POS:
		*(tog->data) = ((input->curspos >= obj->w - 1) ?
				(input->curspos - obj->w + 1) : 0);
		break;
	case TINPUT_HISTORY:
		*(tog->data) = TPDATA(input->text);
		break;
	case TINPUT_HISTORY_COUNT:
		*(tog->data) = input->numhist;
		break;
	case TINPUT_HISTORY_MAXLEN:
		*(tog->data) = input->maxhistory;
		break;
	case TINPUT_CURSOR_POS:
		*(tog->data) = input->curspos;
		break;
	case TINPUT_INPUTMODE:
		*(tog->data) = input->inputmode;
		break;
	case TINPUT_ACCEPT_CHARS:
		*(tog->data) = TPDATA(input->acceptchars);
		break;
	case TINPUT_REJECT_CHARS:
		*(tog->data) = TPDATA(input->rejectchars);
		break;
	default:
		return 0;
	}
	return 1;
}
int tinput_domethod(struct tobj *obj, long method, long data)
{
	struct tinput *input;
	input = (struct tinput *) obj->data;

	switch (method) {
	case TOBJ_SET:
		return tinput_set(obj, (struct tobj_set *) data);
	case TOBJ_GET:
		return tinput_get(obj, (struct tobj_get *) data);
	case TINPUT_CURSOR_LEFT:
		if (input->curspos > 0)
			input->curspos -= 1;
		tobj_dorefresh(obj);
		return 1;
	case TINPUT_CURSOR_RIGHT:
		if (input->curspos < strlen(input->showed))
			input->curspos += 1;
		tobj_dorefresh(obj);
		return 1;
	case TINPUT_CURSOR_BEGIN:
		input->curspos = 0;
		tobj_dorefresh(obj);
		return 1;
	case TINPUT_CURSOR_END:
		input->curspos = strlen(input->showed);
		tobj_dorefresh(obj);
		return 1;
	case TINPUT_PUT_CHAR:
		if (((int) data && (int) data < 256) || 
			((int) data == K_DEL) || ((int) data == K_BS)) {
			tinput_edittext(obj, (int) data);

		}
		return 1;
	case TINPUT_INPUTMODE_SWITCH:
		input->inputmode =
		    (input->inputmode == INSERT) ? REPLACE : INSERT;
		tinput_cursmove(obj);
		return 1;
	}
	return 0;
}
static void tinput_inserthistory(struct tobj *obj)
{
	struct tinput *input;
	input = (struct tinput *) obj->data;


	if (input->maxhistory > 0) {
		int i;
		/* if maxhistory is reached then we free last one */
		if (input->numhist == input->maxhistory) {
			if (input->text[input->numhist])
				free(input->text[input->numhist]);
			input->numhist--;
		}

		/* shifting all entrys in table by one */
		for (i = input->numhist; i >= 0; i--) {
			input->text[i + 1] = input->text[i];
		}


		/* inserting new entry to history */
		if (!(input->text[0] = strdup(input->showed)))
			tsys_error_exit
			    ("ERROR: While inserting new entry to history\n");
		input->numhist++;
	}
}
static void tinput_changebuffers(struct tobj *obj, unsigned int newbuf)
{
	struct tinput *input;
	input = (struct tinput *) obj->data;

	if (newbuf <= input->numhist) {
		input->buf_show = newbuf;

		/* if this buffer was edited just before looking to history
		   (ex. user was editing buffer 1., then looked at buffer 2.,
		   but went back to buffer 1. and continued editing it)
		 */
		if (input->buf_edit == input->buf_show)
			input->showed = input->edited;
		else
			input->showed = input->text[input->buf_show];

		/* correcting cursor pos */
		input->curspos = MIN(input->curspos, strlen(input->showed));
		tobj_dorefresh(obj);
		tevent_send(obj, EVENT_TYPE_USER, EVENT_TINPUT_CHANGED);
	}

}
static int tinput_searchbuffers(struct tobj *obj, char *string)
{
	struct tinput *input;
	unsigned int i, len, f = 0, f_len = 0;

	input = (struct tinput *) obj->data;
	len = strlen(string);

	for (i = 0; i <= input->numhist; i++) {
		if (strncmp(string, input->text[i], len) == 0) {
			unsigned int tmp_len;
			tmp_len = strlen(input->text[i]);
			if (f_len < tmp_len) {
				f = i;
				f_len = tmp_len;
			}
		}
	}

	if (f_len > 0)
		return f;
	else
		return -1;
}
static void tinput_accept(struct tobj *obj)
{
	struct tinput *input;
	input = (struct tinput *) obj->data;


	/* adding edited and accepted string to history */
	tinput_inserthistory(obj);

	input->buf_show = 0;
	input->buf_edit = 0;
	input->showed = input->text[0];
	strcpy(input->edited, input->showed);

	/* Change active object to next one */
	tobj_domethod(obj->parent, TWIN_ACTIVATE_NEXT, 0);

	tevent_send(obj, EVENT_TYPE_USER, EVENT_TINPUT_ACCEPT);
}
static void tinput_reject(struct tobj *obj)
{
	struct tinput *input;
	input = (struct tinput *) obj->data;

	strcpy(input->edited, input->text[input->buf_edit]);
	input->showed = input->text[input->buf_edit];
	input->curspos = MIN(input->curspos, strlen(input->showed));
	tobj_domethod(obj->parent, TWIN_ACTIVATE_NEXT, 0);

	tevent_send(obj, EVENT_TYPE_USER, EVENT_TINPUT_REJECT);
}

int tinput_doevent(struct tobj *obj, int keycode)
{
	struct tinput *input;
	input = (struct tinput *) obj->data;

	switch (keycode) {
	case K_LEFT:
		tinput_domethod(obj, TINPUT_CURSOR_LEFT, 0);
		break;
	case K_RIGHT:
		tinput_domethod(obj, TINPUT_CURSOR_RIGHT, 0);
		break;
	case K_CTL_LEFT:
	case K_HOME:
		tinput_domethod(obj, TINPUT_CURSOR_BEGIN, 0);
		break;
	case K_CTL_RIGHT:
	case K_END:
		tinput_domethod(obj, TINPUT_CURSOR_END, 0);
		break;
	case K_INS:
		tinput_domethod(obj, TINPUT_INPUTMODE_SWITCH, 0);
		break;
	case K_PGUP:
		tinput_changebuffers(obj, input->buf_show + 1);
		break;
	case K_PGDN:
		tinput_changebuffers(obj, input->buf_show - 1);
		break;
	case K_RET:
		tinput_accept(obj);
		break;
	case K_ESC:
		tinput_reject(obj);
		break;
	case K_CTL_RET:
		{
			int i;
			i = tinput_searchbuffers(obj, input->showed);

			/* if founded buffer is the same what was edited */
			if ((unsigned int) i == input->buf_show) {
				strcpy(input->edited,
				       input->text[input->buf_show]);
				tobj_dorefresh(obj);
			} else if (i != -1)
				tinput_changebuffers(obj, i);

			break;
		}
	default:
		/* rest of printable chars */
		if ((keycode >= ' ' && keycode < 256) ||
		    (keycode == K_DEL) || (keycode == K_BS))
			tinput_edittext(obj, keycode);
		else
			/* keycode not used. maybe someone will use it */
			return 0;
		break;
	}
	return 1;
}

void tinput_cleanup(struct tobj *obj)
{
	struct tinput *input;
	unsigned int i = 0;
	input = (struct tinput *) obj->data;

	while (i <= input->maxhistory) {
		if (input->text[i]) {
			free(input->text[i]);
			input->text[i] = NULL;
		}
		i++;
	}
	if (input->text) {
		free(input->text);
		input->text = NULL;
	}
}
void tinput_free(struct tobj *obj)
{
	struct tinput *input;
	input = (struct tinput *) obj->data;

	tinput_cleanup( obj );
	
	if (input) 
		free(input);
}

int tinput_setup(struct tobj *obj, struct tobj_fnc *fnc,
		char *deftext, unsigned int maxlen)
{
	struct tinput *input;
	char nomem[] = "ERROR: While allocating memory for tinput.\n";
	input = (struct tinput *) obj->data;

	if (!(input->text = calloc(10 + 1, sizeof(char *)))) {
		tsys_error(nomem);
		return 0;
	}
	if (!(input->edited = calloc(maxlen + 1, sizeof(char *)))) {
		tsys_error(nomem);
		return 0;
	} else {
		if (maxlen >= strlen(deftext)) {
			if (!(input->text[0] = strdup(deftext))) {
				tsys_error(nomem);
				return 0;
			}
		} else {
			if (!
			    (input->text[0] =
			     malloc(sizeof(char *) * (maxlen + 1)))) {
				tsys_error(nomem);
				return 0;
			}
			strncpy(input->text[0], deftext, maxlen);
			input->text[0][maxlen] = '\0';
		}
		input->maxlen = maxlen;
		input->maxhistory = 10;
		input->numhist = 0;

		input->buf_show = input->buf_edit = 0;
		input->showed = input->edited;
		strcpy(input->edited, input->text[0]);


		input->inputmode = INSERT;
		input->curspos = 0;


		fnc->free = &tinput_free;
		fnc->dorefresh = &tinput_dorefresh;
		fnc->doevent = &tinput_doevent;
		fnc->domethod = &tinput_domethod;

		return 1;
	}
}
struct tobj *tinput_init(struct tobj *parent, unsigned int x,
			 unsigned int y, unsigned int len, char *deftext,
			 unsigned int maxlen)
{
	struct tobj *obj;
	struct tinput *input;
	char nomem[] = "ERROR: While allocating memory for tinput.\n";

	if (!(obj = calloc(1, sizeof(struct tobj)))) {
		tsys_error(nomem);
		return 0;
	}
	if (!(input = calloc(1, sizeof(struct tinput)))) {
		tsys_error(nomem);
		return 0;
	} else {
		obj->data = input;

		obj->x = x;
		obj->y = y;
		obj->h = 1;
		obj->w = len;

		/* setting default values */
                obj->fgcolor = TINPUTFG;
                obj->fgcolor2 = TINPUTFG2;
                obj->bgcolor = TINPUTBG;
                obj->bgcolor2 =TINPUTBG2;
		obj->activeable = 1;
		obj->parent = parent;
		
		if ((tinput_setup( obj , &obj->fnc, deftext , maxlen ) )){
			tobj_init( parent, obj );
			return obj;
		}
		else
			return 0;

	}
}
