#include "tsys.h"
#include "tobj.h"
/* defines of events and methods */
#include "ttool.h"
#include <string.h>

#include "prvclass/ttool.h"

static int ttool_set(struct tobj *obj, struct tobj_set *tos)
{
	struct ttool *tool = (struct ttool *) obj->data;

        switch (tos->attr) {
        case TTOOL_TITLE:
                if (!(tool->title = strdup((char *) tos->data)))
                        tsys_error_exit
                            ("ERROR: While setting tool title.\n");
                tobj_dorefresh(obj);
                break;
        default:
                        return 0;
        }
        return 1;
}

static int ttool_get(struct tobj *obj, struct tobj_get *tog)
{
	struct ttool *tool = (struct ttool *) obj->data;

        switch (tog->attr) {
        case TTOOL_TITLE:
                *(tog->data) = TPDATA(tool->title);
                break;
	
        default:
                return 0;
        }
        return 1;
}

static void ttool_do_fnc(struct tobj *obj)
{
	struct ttool *tool = (struct ttool *) obj->data;
	
	tool->fnc(obj, (long)tool->key);
}

int ttool_domethod(struct tobj *obj, long method, long data)
{
        switch (method) {
        case TOBJ_SET:
                return ttool_set(obj, (struct tobj_set *) data);
        case TOBJ_GET:
                return ttool_get(obj, (struct tobj_get *) data);
	case TTOOL_DO_FNC:
		ttool_do_fnc(obj);
		break;
        }
        return 1;
}

int ttool_doevent(struct tobj *obj, int keycode)
{
	struct ttool *tool = (struct ttool *) obj->data;

	if (tool->key == keycode) {
		ttool_do_fnc(obj);
		
	} else
		return 0;
        return 1;
}

void ttool_cleanup(struct tobj *obj)
{
	struct ttool *tool = (struct ttool *) obj->data;

        if (tool->title)
                free(tool->title);

}
void ttool_free(struct tobj *obj)
{
	ttool_cleanup(obj);
}

int ttool_setup(struct tobj *obj, notifyfnc fnc,
	int key, char *title, long state)
{
        struct ttool *tool = (struct ttool *) obj->data;
        char nomem[] = "ERROR: While allocating memory for ttool.\n";

	tool->fnc = fnc;
	tool->key = key;
	tool->state = state;
        if (title) {
                if (!(tool->title = strdup(title))) {
                        tsys_error(nomem);
                        return 0;
                }
        } else
                tool->title = NULL;

        return 1;
}

struct tobj *ttool_init(struct tobj *parent, notifyfnc fnc, int key,
	char *title, long state)
{
        struct tobj *obj;
        struct ttool *tool;
        char nomem[] = "ERROR: While allocating memory for ttool.\n";

        if (!(obj = calloc(1, sizeof(struct tobj)))) {
                tsys_error(nomem);
                return 0;
        }
        if (!(tool = calloc(1, sizeof(struct ttool)))) {
                tsys_error(nomem);
                return 0;
        } else {
                obj->data = tool;
                obj->x = 0;
                obj->y = 0;
                obj->h = 0;
                obj->w = 0;

                /* setting default values */
                obj->fgcolor2 = BLACK;
                obj->fgcolor = WHITE;
		obj->bgcolor = LIGHTGRAY;
                obj->bgcolor2 = GREEN;

                if (title)
                        obj->activeable = 1;

                if ((ttool_setup(obj, fnc, key, title, state))) {
                        tobj_init(parent, obj);
			obj->next = 0;
                        return obj;
                } else
                        return 0;
        }
}
